/*
  ProgEx40   -- Form3: Volume Of Your Box.  Decorating Your Form With User Interface
                Controls Such As Edit Boxes, Buttons, And Combo Boxes.

  I'm going to try something different.  Usually when I write tutorials I come up with an
  idea for a program; I write the whole program; then I post the whole thing with comments
  and discussion throughout.  In this case, I have the idea for a good learning program,
  but I'll post it in segments as it develops.  In that way you'll hopefully be able to
  grasp the individual pieces as it develops, rather than being intimidated by all the
  code bulk of the final product.

  Lets start with something ridiculously simple.  My idea is a form with several text
  boxes where one can enter the length, width, and height of a box.  There will be two
  buttons on it.  The 1st button will calculate the volume of the box and display it in
  another text box labeled 'Volume'.  The 2nd button named 'Display' will output the entry
  data and volume to a programatically created output screen which the user can dismiss
  when finished viewing the output.  Finally, the main Form will have a drop-down combo
  box near the top with several selections in it regarding the kind of box we are working
  with.

  We already have a template program we can start with, and that would be Form1 of
  ProgEx39c.  Here are some of the things I did to that example so it could be used for
  this little project;

  1) Remove all the MessageBoxes;

  2) Changed the TCHARs to char.  Lets not kill ourselves over that UNICODE nonesense;

  3) Replace the WM_LBUTTONDOWN message handling with a WM_COMMAND handler;

  4) Add a bit more complexity to the message loop down in WinMain() to deal with tab
     order of child window controls on the Main Form.  We need to use IsDialogMessage()
     for that;

  5) Add code in WM_CREATE to create our child window controls, i.e., text boxes, buttons,
     etc.

  Believe it or not, that's it!  Take a look at the code below and compare it with
  ProgEx29c and you'll see I speak the truth.  Better yet, compile the code and run it and
  you should see a nice little program that at this point doesn't calculate anything, but
  at least most of the user interface is working and the tab order works.

  Of course, fnWndProc_OnCreate() is pretty unrecognizable.  That's where some big time
  stuff happened.  Lets take it one line and one concept at a time.

  This line...

  char* szBuffer[]={"Ordinary Box","Box Of Junk","Beer Crate","Wine Box","Candy Box"};

  creates a character string array of the text strings that are to go in our combo-box.

  This line...

  DWORD dwStyle=WS_CHILD|WS_VISIBLE|WS_TABSTOP;

  creates some useful Window Style bits for inclusion in the many CreateWindow() and
  CreateWindowEx() calls below.  Here momentarily we'll discuss bits and Window Styles.
  This line...

  HWND hCtl;

  declares an object of HWND (handle to a window) that will be returned from the various
  CreateWindow() calls of our child control creation.

  Right at the top of the WM_CREATE handler you see this as the 1st line of code...

  Wea->hIns=((LPCREATESTRUCT)Wea->lParam)->hInstance;

  If you look down in WinMain() at the parameter list to the function you'll see that
  something termed a...

  HINSTANCE hInstance

  was the 1st parameter in the parameter list, and that it was used in several places
  there.  Most importantly for us here, it is the next to last parameter of the big
  CreateWindow() call.  However, its essentially a local variable in WinMain() and we're
  working under the rather strict regimen here of not allowing ourselves global variables.
  So how are we going to get it?  Well, as it turns out there is no end of ways to get
  it.

  When you make a CreateWindow() call Windows stores all the many parameters of the call
  internally in something named a CREATESTRUCT...

  typedef struct tagCREATESTRUCT
  { // cs
    LPVOID    lpCreateParams;
    HINSTANCE hInstance;
    HMENU     hMenu;
    HWND      hwndParent;
    int       cy;
    int       cx;
    int       y;
    int       x;
    LONG      style;
    LPCTSTR   lpszName;
    LPCTSTR   lpszClass;
    DWORD     dwExStyle;
  }CREATESTRUCT;

  Note that the 2nd member is the HINSTANCE of the application.  And when Windows sends a
  WM_CREATE message, the lParam is a pointer to this CREATESTRUCT.  In terms of the
  awkward syntax with the casting, you'll just have to learn to live with it.  Afterall,
  we're doing C/C++ here!  Its not supposed to be nice or friendly!  In PowerBASIC it just
  looks like this...

  Local lpCreateStruct As CREATESTRUCT Ptr
  lpCreateStruct=Wea.lParam
  Wea.hInst=@lpCreateStruct.hInstance

  Anyway, the HINSTANCE could also be gotten using...

  Wea->hIns=GetModuleHandle(NULL);

  or even...

  Wea->hIns=(HINSTANCE)GetWindowLong(Wea->hWnd,GWL_HINSTANCE);

  Enough said about that I guess!  So now we're at our 1st CreateWindow() call!  And for a
  lowly label no less!

  hCtl=
  CreateWindow
  (
    "static",               //Class name of Window to be instantiated
    "Choose Kind of Box",   //Caption if any
    WS_CHILD|WS_VISIBLE,    //Window styles desired bit wise or'ed together
    10,                     //x pixel location desired of window
    10,                     //y pixel --------------------------
    140,                    //Width of Window
    25,                     //Height --------
    Wea->hWnd,              //Parent of Window or 0=HWND_DESKTOP if top level main Window
    (HMENU)-1,              //Either menu identifier or Control ID if child Window
    Wea->hIns,              //Process instance handle, i.e., hInstance
    0                       //Pointer to Creation Parameters if any or NULL
  );

  You may have noted that sometimes I use CreateWindow() and at other times
  CreateWindowEx(). Truth be told I should probably always use CreateWindowEx() because it
  superceded CreateWindow() some time ago.  However, the only difference between the two
  is that an extra Window Style parameter was added named dwExStyle.  For the label
  control above the extended styles don't do much so I just used CreateWindow().  Had I
  used CreateWindowEx() in the call above I'd have just put a zero in the unused parameter
  place.

  Since these calls figure pretty prominently in the code we're discussing, here is the
  Api help on CreateWindowEx()...

  HWND CreateWindowEx
  (
    DWORD dwExStyle,      // extended window style
    LPCTSTR lpClassName,  // pointer to registered class name
    LPCTSTR lpWindowName, // pointer to window name
    DWORD dwStyle,        // window style
    int x,                // horizontal position of window
    int y,                // vertical position of window
    int nWidth,           // window width
    int nHeight,          // window height
    HWND hWndParent,      // handle to parent or owner window
    HMENU hMenu,          // handle to menu, or child-window identifier
    HINSTANCE hInstance,  // handle to application instance
    LPVOID lpParam        // pointer to window-creation data
  );

  Have you figured out yet that to do this kind of coding you need to constantly have a
  reference to the Windows Api functions handy?  I bet you have!

  The "static" term in our 1st child window CreateWindow() call refers to the class name
  for a label.  If you check out the Api documentation on CreateWindow() or
  CreateWindowEx() you'll find all the various class names.  The only ones we're using
  here are "static", "button", "edit", and "combobox".

  In case you're curious, note that in WinMain() we Registered with Windows our own class
  name when we filled out the members of a WNDCLASSEX struct and registered it with
  RegisterClassEx().  Back there we used the class name "Form3" and in our CreateWindow()
  call we put "Form3" in the parameter location we're putting "static", "edit", etc.,
  here.  With regard to this, pause for a moment and think about the big picture of what's
  going on here.  When we filled out those WNDCLASSEX fields of that struct down in
  WinMain() and registered that class, one of the fields we filled out was the pointer to
  the Window Procedure.  And we're now coding more CreateWindow() calls in the WM_CREATE
  handler of that Window Procedure for the Form3 Window - our main program window.  Has it
  occurred to you yet as to where the Window Procedure might be for these "static",
  "button", etc., windows we're here creating?  Search as you may you won't find them here.
  These are controls predefined by Microsoft and the Window Procedures for these controls
  are within Windows itself.  We don't have to create buttons, edit controls, etc.,
  ourselves.  Microsoft has already done it.  However, if we wish to build a main
  application window or a custom control in which are located other simpler controls and
  which has its own Window Procedure, we are free to do so.

  Lets move on to the Style parameter of the CreateWindow() call.  This is quite important
  and in the Windows Api documentation you'll find myriads of styles for the various
  controls and quite a few for even top level main program windows.  For our lowly static
  control, i.e., label, the ones most frequently used are WS_CHILD | WS_VISIBLE.  If you
  look down in WinMain our main Form3 program window used a style of WS_OVERLAPPEDWINDOW.
  In this latter case WS_OVERLAPPEDWINDOW is a shorthand composite style comprised of...

  WS_OVERLAPPED, WS_CAPTION, WS_SYSMENU, WS_THICKFRAME, WS_MINIMIZEBOX, and WS_MAXIMIZEBOX

  Moving on to the 4th, 5th, 6th, and 7th parameters of our 1st CreateWindow() call in
  fnWndProc_OnCreate(), these are nothing more than the x/y pixel coordinate position
  where we want the upper left hand corner of our window instantiated with regard to its
  parent, and the pixel width/height desired for the window.

  The 8th parameter is the HWND of the parent of the window.  For our label its the main
  program window represented by Wea->hWnd (at this point you're probably tired of handles.
  After all, what are they?  Best description would probably be opaque pointers.  They
  reference private data internal to windows that we Microsoft outsiders don't need to
  know about).  Down in WinMain() for our main program window we just used HWND_DESKTOP
  which is an equate equal to zero (in other words, it doesn't have a parent, or the
  parent is the desktop).

  The next parameter - the 9th, is a menu handle for the window if it has a menu or a
  child window numeric identifier.  The latter applies in our case as the label doesn't
  have a menu nor even our main program window.  However, the parameter is typed as taking
  a handle to a menu ( HMENU ) which is a specific Windows data type defined in Windows
  headers, so we need the HMENU cast you see.  And in the case of labels the numeric value
  -1 can be used telling Windows to not bother tracking the child window id of the
  control.  However, this is a very important topic even though in this specific instance
  we're more or less ignoring the issue with the (HMENU)-1 value.  For you see, all child
  windows such as buttons, edit boxes, etc., need an integral numeric value associated
  with the specific instance of the control - whatever it may be.  Please take a look at
  ProgEx40.h and right near the top you'll see this...

  #define IDC_COMBO_TYPE          1500
  #define IDC_EDIT_LENGTH         1505
  #define IDC_EDIT_WIDTH          1510
  #define IDC_EDIT_HEIGHT         1515
  #define IDC_EDIT_VOLUME         1520
  #define IDC_BUTTON_CALCULATE    1525
  #define IDC_BUTTON_DISPLAY      1530

  I've assigned numeric values starting at 1500 and incrementing by 5 for all the controls
  on the main form ( Form3 ) except for the label controls adjacent to the combo box and
  the four edit controls.  All this relates pretty closely to the fact that when I created
  all the child window controls with no less than 12 CreateWindow() or CreateWindowEx()
  Api calls I disgarded each and every returned hCtl from each function call rather than
  saving it in a specifically declared unique HWND variable such as hWndLength for the
  Window Handle of the Length textbox; hWndWidth for the Width textbox, etc.  I put each
  one in hCtl and it was immediately lost with the next call.  The reason I did this is
  that if you know the child control id of a control and its parent, you can always
  retrieve the control's HWND from...

  hWndChildControl = GetDlgItem(hParent, CHILD_WINDOW_CONTROL_ID);

  Window handles are an extremely important and needed variable in Windows programming,
  but you never need make global variables out of them for this reason.  In this sense,
  you can think of these child window control ids as proxies for Window Handles.  They are
  global elements with the attribute of being equeate/defines which aren't altered.  In
  that sense I wouldn't have even needed assigning the return to each CreateWindow() call
  to hCtl; I could have just did a bare CreateWindow() call as a sub-routine without a
  return; however, I'm used to seeing return values on CreateWindow() call.

  The next parameter - the 10th, is the HINSTANCE we needed and got from the lParam using
  the CREATESTRUCT pointer.  This would be the same HINSTANCE that came through in the 1st
  parameter of the WinMain() call and was used there for our main Window.

  The last parameter - the 11th, we just assigned NULL.  We didn't use it.  However its
  typed like so...

  LPVOID lpParam        // pointer to window-creation data

  This is a pointer parameter and it can be a pointer to anything whatsoever you have a
  mind to pass to the WM_CREATE handler of your Window Procedure.  Its another global
  killer is what it is.  I'm not going to say anymore about it here because we're not
  using it in this first version of this app.  Just tuck away for future reference that
  its very important (and we'll be using it soon).

  Whoa!  That was a lot of description for a lowly label/static control!  I'd better speed
  things up and tell you that the above descriptions apply in a general sense to all the
  other CreateWindow() calls!  You'll note in all the other calls that aren't labels the
  control id defines from ProgEx40.h are placed in the HMENU/Control Id parameter
  position.  Also, the various types of controls have the required styles to give them the
  functionality they need.  For example, I used CreateWindowEx() calls for the edit
  controls because I wanted a nice 3D sunken edge effect for them, and for that I needed
  this extended style...

  WS_EX_CLIENTEDGE

  Also, I wanted some sort of reasonable tabbing functionality so I included the
  WS_TABSTOP style in all the non-label controls as well as styles to make them child
  controls that were visible.

  The last major thing I want to cover in this 1st installment of Form3 is the
  SendMessage() call that put our box types in the combo box.  This is a really important
  Windows Api call.  First note (pull up MSDN or your Api Docs on SendMessage) that the
  parameters of the call exactly match the function signature of a Window Procedure.  Here
  I'm not speaking of my event handlers such as fnWndProc_OnCreate() or so on and so forth
  but the main program's Window Procedure - fnWndProc(), whose signature is thus...

  HWND hwnd, unsigned int msg, WPARAM wParam, LPARAM lParam

  In object oriented programming objects communicate with each other by sending messages
  to one another (Yes, we're doing a kind of OOP here even though we're essentially using
  C manerisms to do it).  In this sequence of code...

  for(unsigned int i=0; i<5; i++)
      SendMessage(hCtl,CB_INSERTSTRING,(WPARAM)-1,(LPARAM)szBuffer[i]);

  ...the hCtl parameter is the Window Handle for the just created Combo Box.  Note I put
  this loop that uses the HWND of the Combo Box right after the CreateWindow() call that
  created the Combo Box because I knew I'd lose the HWND after the next CreateWindow()
  call so I wanted to use it while I still had it.  Now do a search in your documentation
  for either Combo Box, Combo Box Messages, Default Combo Box Behavior or whatever until
  you come up with a reference to Combo Box specific messages.  There are a lot of them
  and examples would be CB_ADDSTRING, CB_DELETESTRING, CB_INSERTSTRING, CB_GETCOUNT, etc.
  If you do a search for CB_INSERTSTRING you'll eventually come to this...

                             --Microsoft Documentation--

  CB_INSERTSTRING

  An application sends a CB_INSERTSTRING message to insert a string into the list box of a
  combo box. Unlike the CB_ADDSTRING message, the CB_INSERTSTRING message does not cause a
  list with the CBS_SORT style to be sorted.

  CB_INSERTSTRING

  wParam = (WPARAM) index;          // item index
  lParam = (LPARAM) (LPCTSTR) lpsz; // address of string to insert

  Parameters

  index   Value of wParam. Specifies the zero-based index of the position at which to
          insert the string. If this parameter is 1, the string is added to the end of
          the list.

  lpsz    Value of lParam. Pointer to the null-terminated string to be inserted. If you
          create the combo box with an owner-drawn style but without the CBS_HASSTRINGS
          style, the value of the lpsz parameter is stored rather than the string it would
          otherwise point to.

  Return Values

  The return value is the index of the position at which the string was inserted. If an
  error occurs, the return value is CB_ERR. If there is insufficient space available to
  store the new string, it is CB_ERRSPACE.

                                   --End Microsoft--

  So, you can see where the other parameters came from.  Of course, if we were doing this
  in PowerBASIC we wouldn't need the silly casts...

  For i=0 To 4
    Call SendMessage(hCtl, %CB_INSERTSTRING, -1, szBuffer(i))
  Next i

  ...but that's another story!  But the important point to grasp here is that a Combo Box
  is a very specific Window Class designed by Microsoft, and its Window Procedure the code
  of which imbues it with all the behaviors we've come to expect of Combo Boxes, is within
  Windows Operating System Dlls and we communicate with that code as here with SendMessage
  function calls.  This is in fact a generalized mechanism for the interaction of
  controls, i.e., windows, throughout the operating system, and the function signature of
  Window Procedures and SendMessage calls is something you'll run into in many places.

  Finally, near the bottom of fnWndProc_OnCreate I use SetWindowText() to alter the
  caption of the main program window ( Form3 ), and SetFocus() to set the focus on the
  top most control on the Form which is the Box Type Combo Box.  Note in that call I used
  the GetDlgItem() call to retrieve the HWND of the Combo Box which I so foolishly and in
  such a cavalier manner disgarded.

  In the next version of this program we'll flesh out some of the additional functionality
  we need to start getting things to work when we type text and click buttons.
*/

//Main.cpp
#include <windows.h>
#include "ProgEx40.h"
EVENTHANDLER  EventHandler[3];


long fnWndProc_OnCreate(lpWndEventArgs Wea)
{
 char* szBuffer[]={"Ordinary Box","Box Of Junk","Beer Crate","Wine Box","Candy Box"};
 DWORD dwStyle=WS_CHILD|WS_VISIBLE|WS_TABSTOP;
 HWND hCtl;

 Wea->hIns=((LPCREATESTRUCT)Wea->lParam)->hInstance;
 hCtl=CreateWindow("static","Choose Kind of Box",WS_CHILD|WS_VISIBLE,10,10,140,25,Wea->hWnd,(HMENU)-1,Wea->hIns,0);
 hCtl=CreateWindow("combobox","",dwStyle|CBS_DROPDOWNLIST|WS_VSCROLL,150,10,140,100,Wea->hWnd,(HMENU)IDC_COMBO_TYPE,Wea->hIns,0);
 for(unsigned int i=0; i<5; i++)
     SendMessage(hCtl,CB_INSERTSTRING,(WPARAM)-1,(LPARAM)szBuffer[i]);
 SendMessage(hCtl,CB_SETCURSEL,(WPARAM)0,0);  //|WS_TABSTOP
 hCtl=CreateWindow("static","Length of Box",WS_CHILD|WS_VISIBLE,10,53,125,25,Wea->hWnd,(HMENU)-1,Wea->hIns,0);
 hCtl=CreateWindowEx(WS_EX_CLIENTEDGE,"edit","",dwStyle,150,50,70,25,Wea->hWnd,(HMENU)IDC_EDIT_LENGTH,Wea->hIns,0);
 hCtl=CreateWindow("static","Width of Box",WS_CHILD|WS_VISIBLE,10,93,125,25,Wea->hWnd,(HMENU)-1,Wea->hIns,0);
 hCtl=CreateWindowEx(WS_EX_CLIENTEDGE,"edit","",dwStyle,150,90,70,25,Wea->hWnd,(HMENU)IDC_EDIT_WIDTH,Wea->hIns,0);
 hCtl=CreateWindow("static","Height of Box",WS_CHILD|WS_VISIBLE,10,133,125,25,Wea->hWnd,(HMENU)-1,Wea->hIns,0);
 hCtl=CreateWindowEx(WS_EX_CLIENTEDGE,"edit","",dwStyle,150,130,70,25,Wea->hWnd,(HMENU)IDC_EDIT_HEIGHT,Wea->hIns,0);
 hCtl=CreateWindow("static","Volume",WS_CHILD|WS_VISIBLE,10,183,70,25,Wea->hWnd,(HMENU)-1,Wea->hIns,0);
 hCtl=CreateWindowEx(WS_EX_CLIENTEDGE,"edit","",dwStyle,80,180,300,25,Wea->hWnd,(HMENU)IDC_EDIT_VOLUME,Wea->hIns,0);
 hCtl=CreateWindow("button","Calculate",dwStyle,245,65,120,30,Wea->hWnd,(HMENU)IDC_BUTTON_CALCULATE,Wea->hIns,0);
 hCtl=CreateWindow("button","Display",dwStyle,245,110,120,30,Wea->hWnd,(HMENU)IDC_BUTTON_DISPLAY,Wea->hIns,0);
 SetWindowText(Wea->hWnd,"Volume Of Your Box");
 SetFocus(GetDlgItem(Wea->hWnd,IDC_COMBO_TYPE));

 return 0;
}

long fnWndProc_OnCommand(lpWndEventArgs Wea)
{
 return 0;
}


long fnWndProc_OnClose(lpWndEventArgs Wea)
{
 DestroyWindow(Wea->hWnd);
 PostQuitMessage(0);
 return 0;
}


void AttachEventHandlers(void)         //This procedure maps windows messages to the
{                                      //procedure which handles them.
 EventHandler[0].Code=WM_CREATE,       EventHandler[0].fnPtr=fnWndProc_OnCreate;
 EventHandler[1].Code=WM_COMMAND,      EventHandler[1].fnPtr=fnWndProc_OnCommand;
 EventHandler[2].Code=WM_CLOSE,        EventHandler[2].fnPtr=fnWndProc_OnClose;
}


long __stdcall fnWndProc(HWND hwnd, unsigned int msg, WPARAM wParam,LPARAM lParam)
{
 WndEventArgs Wea;                  //This procedure loops through the EVENTHANDER array
                                    //of structs to try to make a match with the msg parameter
 for(unsigned int i=0; i<3; i++)    //of the WndProc.  If a match is made the event handling
 {                                  //procedure is called through a function pointer -
     if(EventHandler[i].Code==msg)  //(EventHandler[i].fnPtr).  If no match is found the
     {                              //msg is passed onto DefWindowProc().
        Wea.hWnd=hwnd, Wea.lParam=lParam, Wea.wParam=wParam;
        return (*EventHandler[i].fnPtr)(&Wea);
     }
 }

 return (DefWindowProc(hwnd, msg, wParam, lParam));
}


int __stdcall WinMain(HINSTANCE hInstance, HINSTANCE hPrevIns, LPSTR lpszArgument, int iShow)
{
 char szClassName[]="Form3";
 WNDCLASSEX wc;
 MSG messages;
 HWND hWnd;

 AttachEventHandlers();
 wc.lpszClassName=szClassName;                wc.lpfnWndProc=fnWndProc;
 wc.cbSize=sizeof (WNDCLASSEX);               wc.style=CS_DBLCLKS;
 wc.hIcon=LoadIcon(NULL,IDI_APPLICATION);     wc.hInstance=hInstance;
 wc.hIconSm=LoadIcon(NULL, IDI_APPLICATION);  wc.hCursor=LoadCursor(NULL,IDC_ARROW);
 wc.hbrBackground=(HBRUSH)COLOR_BTNSHADOW;    wc.cbWndExtra=0;
 wc.lpszMenuName=NULL;                        wc.cbClsExtra=0;
 RegisterClassEx(&wc);
 hWnd=CreateWindowEx(0,szClassName,szClassName,WS_OVERLAPPEDWINDOW,200,400,400,250,HWND_DESKTOP,0,hInstance,0);
 ShowWindow(hWnd,iShow);
 while(GetMessage(&messages,NULL,0,0))
 {     //Is it not a Dialog Message?
    if(!IsDialogMessage(hWnd,&messages)) //Added this because it does special character code translations that
    {                                    //allows the use of the tab key to move focus among child window
       TranslateMessage(&messages);      //controls with the WS_TABSTOP style on the main Form.
       DispatchMessage(&messages);
    }
 }

 return messages.wParam;
}
